using System.Collections.Generic;

using AntMe.English;


//Insert a slashdot behind AntMe.Player followed by your name (please don't use blanks).
//It could look like this: "AntMe.Player.NickBoesl".
namespace AntMe.Player.hriss95
{
    //Using the 'player'-attribute you can specify the name of your ant colony and the 
    //first and last name of the player. In order for the game to find your ant colony you
    //have to name your colony; inserting your name 
    [Player(
        ColonyName = "hriss95",
        FirstName = "Hristian",
        LastName = "Hristov"
        )]

    //The 'caste'-attribute makes it possible to change the properties of your ants. You can make 
    //them faster or give them more energy etc. But be careful: Only '-1', '0', '1' and '2' can be
    //be used and the sum of all properties of one caste has to be '0'!
    //To use this attribute you have to name the caste fist. After that you can return the name of that
    //type by using 'ChooseType'.
    //For further information how to use the ant-types see chapter 6 of the tutorial. 
    [Caste(
        Name = "Fighter",
        SpeedModificator = 0,
        LoadModificator = -1,
        AttackModificator = 2,
        EnergyModificator = 2,
        RangeModificator = -1,
        ViewRangeModificator = -1,
        RotationSpeedModificator = -1
        )]
    [Caste(
        Name = "Gatherer",
        SpeedModificator = 1,
        LoadModificator = 2,
        AttackModificator = 0,
        EnergyModificator = 0,
        RangeModificator = -1,
        ViewRangeModificator = -1,
        RotationSpeedModificator = -1
        )]
    [Caste(
        Name = "Scout",
        SpeedModificator = 0,
        LoadModificator = -1,
        AttackModificator = -1,
        EnergyModificator = -1,
        RangeModificator = 2,
        ViewRangeModificator = 2,
        RotationSpeedModificator = -1
        )]
    [Caste(
        Name = "Caller",
        SpeedModificator = -1,
        LoadModificator = -1,
        AttackModificator = 0,
        EnergyModificator = 2,
        RangeModificator = 2,
        ViewRangeModificator = -1,
        RotationSpeedModificator = -1
        )]
    [Caste(
        Name = "Guard",
        SpeedModificator = -1,
        LoadModificator = -1,
        AttackModificator = -1,
        EnergyModificator = 2,
        RangeModificator = 0,
        ViewRangeModificator = 2,
        RotationSpeedModificator = -1
        )]

    /*public static class global
     {
         static int _weak = 0, _strong = 0;

         public static int weak
         {
             get
             {
                 return _weak;
             }
             set
             {
                 _weak = value;
             }
         }
         public static int strong
         {
             get
             {
                 return _strong;
             }
             set
             {
                 _strong = value;
             }
         }
     }

         */
    public class Aggresive_Ant : BaseAnt
    {
        private double coef;

        #region Castes

        /// <summary>
        /// Chooses a specific caste.
        /// </summary>
        /// <param name="typeCount">List of available ants from caste</param>
        /// <returns>Name of chosen caste</returns>
        public override string ChooseType(Dictionary<string, int> typeCount)
        {
            /*if (global.weak >= global.strong) coef = 2;
            else coef = 1.6;*/

            if (typeCount["Scout"] < 4) return "Scout";
            //if (typeCount["Caller"] < 2) return "Caller";
            if (typeCount["Guard"] == 0) return "Guard";
            else if (typeCount["Gatherer"] == 0) return "Gatherer";
            else if (typeCount["Gatherer"] / typeCount["Guard"] >= 4.5) return "Guard";
            else if (typeCount["Fighter"] / typeCount["Gatherer"] > 2.2) return "Gatherer";
            else return "Fighter";
        }

        #endregion

        #region Movement


        /// <summary>
        /// Is called periodically everytime the ant does not know where to go.
        /// </summary>
        public override void Waits()
        {
            if (Caste != "Guard")
            {
                if (CurrentLoad != 0) GoBackToAnthill();
                if (CarringFruit != null) GoBackToAnthill();
                if (IsTired) GoBackToAnthill();
            }
            GoAhead();
        }

        /// <summary>
        /// Is called once when the ant has exceeded a third of it's maximum range.
        /// </summary>
        public override void BecomesTired()
        {
            if (!(Target is Fruit) && !(Target is Sugar)) GoBackToAnthill();
        }

        #endregion

        #region Food

        /// <summary>
        /// Is called periodically everytime the ant spots at least one heap of sugar.
        /// </summary>
        /// <param name="sugar">The closest heap of sugar.</param>
        public override void Spots(Sugar sugar)
        {
            if (Caste == "Guard")
            {
                GoToTarget(sugar);
            }
            if (Caste == "Gatherer" || Caste == "Scout")
            {
                int distance, direction;
                distance = Coordinate.GetDistanceBetween(this, sugar);
                direction = Coordinate.GetDegreesBetween(this, sugar);
                MakeMark(direction, distance);
                if (CurrentLoad == 0 && Caste == "Gatherer" && Target == null)
                {
                    GoToTarget(sugar);
                }
            }
        }

        /// <summary>
        /// Is called periodically everytime the ant spots at least one piece of fruit.
        /// </summary>
        /// <param name="fruit">The closest piece of fruit.</param>
        public override void Spots(Fruit fruit)
        {
            if (Caste == "Gatherer" || Caste == "Fighter" || Caste == "Scout")
            {
                int distance, direction;
                distance = Coordinate.GetDistanceBetween(this, fruit);
                direction = Coordinate.GetDegreesBetween(this, fruit);
                MakeMark(direction, distance);
                if (CurrentLoad == 0 && NeedsCarrier(fruit) && Caste == "Gatherer")
                {
                    GoToTarget(fruit);
                }
            }
        }

        /// <summary>
        /// Is called once when a sugar heap is the ant's target and it reaches the sugar.
        /// </summary>
        /// <param name="sugar">The heap of sugar.</param>
        public override void TargetReached(Sugar sugar)
        {
            if (Caste != "Guard")
            {
                Take(sugar);
                GoBackToAnthill();
            }
        }

        /// <summary>
        /// Is called once when a piece of fruit is the ant's target and it reaches the fruit.
        /// </summary>
        /// <param name="fruit">The piece of fruit.</param>
        public override void TargetReached(Fruit fruit)
        {
            Take(fruit);
            GoBackToAnthill();
        }

        #endregion

        #region Communication

        /// <summary>
        /// Is called once when the ant 'smells' a marker from its own ant colleagues.
        /// A marker can only be 'smelled' once and will never be smelled again.
        /// </summary>
        /// <param name="marker">Closest new marker.</param>
        public override void SmellsFriend(Marker marker)
        {
            if (Caste == "Fighter" && Target == null)
            {
                if (marker.Information >= 720)
                {
                    TurnToDirection(marker.Information);
                    GoAhead();
                }
            }

            if (Caste == "Fighter" && Target == null)
            {
                if (marker.Information >= 360)
                {
                    TurnToDirection(marker.Information - 360);
                    GoAhead();
                }
            }
            if (Caste == "Gatherer" && Target == null)
            {
                if (marker.Information < 360)
                {
                    TurnToDirection(marker.Information);
                    GoAhead();
                }
            }
        }

        /// <summary>
        /// Is called repeatedly each time at least one other ant from the same colony is spotted.
        /// </summary>
        /// <param name="ant">The closest friendly ant.</param>
        public override void SpotsFriend(Ant ant)
        {
        }

        /// <summary>
        /// Is called, when the ant spots a friendly/allied ant from another team.
        /// </summary>
        /// <param name="ant"></param>
        public override void SpotsPartner(Ant ant)
        {
        }

        #endregion

        #region Fight

        /// <summary>
        /// Is called repeatedly each time the ant spots at least one bug.
        /// </summary>
        /// <param name="bug">The closest bug.</param>
        public override void SpotsEnemy(Bug bug)
        {
            if (Caste == "Scout") GoAwayFromTarget(bug);
            if (Caste == "Fighter")
            {
                int distance, direction;
                distance = Coordinate.GetDistanceBetween(this, bug);
                direction = Coordinate.GetDegreesBetween(this, bug);
                MakeMark(direction + 360, distance);
                if (FriendlyAntsFromSameCasteInViewrange * MaximumEnergy > bug.CurrentEnergy * 0.4)
                    Attack(bug);
                else GoAwayFromTarget(bug);
            }
            if (Caste == "Gatherer")
            {
                if (FriendlyAntsInViewrange * MaximumEnergy > bug.CurrentEnergy * 0.6)
                {
                    if (CurrentLoad != 0 && CarringFruit == null) Drop();
                    if (CarringFruit == null) Attack(bug);
                }
            }
        }

        /// <summary>
        /// Is called repeatedly each time the ant spots at least one ant of another colony.
        /// </summary>
        /// <param name="ant">The closest inimical ant.</param>
        public override void SpotsEnemy(Ant ant)
        {
            /* if (Target == null)
             {
                 if (ant.CurrentEnergy > 170) global.weak++;
                 else global.strong++;
             }*/
            if (Caste == "Guard")
            {
                int distance, direction;
                distance = Coordinate.GetDistanceBetween(this, ant);
                direction = Coordinate.GetDegreesBetween(this, ant);
                MakeMark(direction + 360, distance + 150);
            }
            if (Caste == "Scout") GoAwayFromTarget(ant);
            if (Caste == "Fighter")
            {
                int distance, direction;
                distance = Coordinate.GetDistanceBetween(this, ant);
                direction = Coordinate.GetDegreesBetween(this, ant);
                MakeMark(direction + 360, distance);
                if (FriendlyAntsFromSameCasteInViewrange * MaximumEnergy > ForeignAntsInViewrange * ant.CurrentEnergy)
                    Attack(ant);
                else GoAwayFromTarget(ant);
            }
        }

        /// <summary>
        /// Is called repeatedly each time the ant is attacked by a bug.
        /// </summary>
        /// <param name="bug">Closest attacking bug.</param>
        public override void UnderAttack(Bug bug)
        {
            if (Caste == "Scout") GoAwayFromTarget(bug);
            if (Caste == "Fighter")
            {
                if (FriendlyAntsFromSameCasteInViewrange * MaximumEnergy < bug.CurrentEnergy)
                    Attack(bug);
                else GoAwayFromTarget(bug);
            }
        }

        /// <summary>
        /// Is called repeatedly each time the ant is attacked by another ant.
        /// </summary>
        /// <param name="ant">The closest inimical attacking ant.</param>
        public override void UnderAttack(Ant ant)
        {
            if (Caste == "Scout") GoAwayFromTarget(ant);
            if (Caste == "Fighter")
            {
                if (FriendlyAntsFromSameCasteInViewrange < ForeignAntsInViewrange) GoAwayFromTarget(ant);
                else Attack(ant);
            }
        }

        #endregion

        #region Misc

        /// <summary>
        /// Is called when the ant has just died.
        /// </summary>
        /// <param name="kindOfDeath">The reason for the ant's death.</param>
        public override void HasDied(KindOfDeath kindOfDeath)
        {
        }

        /// <summary>
        /// Is called each simulation step and not inflicted by any other external influence.
        /// </summary>
        public override void Tick()
        {
            /*if (global.weak + global.strong > 150)
            {
                global.weak = 0;
                global.strong = 0;
            }*/
            if (CurrentLoad > 0 && Target == null && CarringFruit == null) GoBackToAnthill();
            if (Target is Anthill && CurrentLoad > 0 && CarringFruit == null)
            {
                GoBackToAnthill();
                MakeMark(Direction + 180);
            }
            if (CarringFruit != null) GoBackToAnthill();
        }

        #endregion
    }
}